(function() {
'use strict';

if (typeof Proxy === 'function') {
    console.log = new Proxy(console.log, {
        apply: (target, thisArg, args) => {
            target.apply(thisArg, args);

            const li = document.createElement('li');
            li.textContent = args.join(' ');
            const wrapper = document.getElementById('console-list');
            wrapper.appendChild(li);
        }
    });
}

const CODES = {
    ENTER: 13,
    ESC:  27,
    SPACE: 32,
    LEFT: 37,
    UP: 38,
    RIGHT: 39,
    DOWN: 40,
};

function dispatchClickEvent(e) {
    const clickEvent = new MouseEvent('click', {
        'view': window,
        'bubbles': true,
        'cancelable': true
    });
    e.currentTarget.dispatchEvent(clickEvent);
}


class MenuItem {
    constructor(domElement, parent, index) {
        this.domElement = domElement;
        this.parent = parent;
        this.init();
    }

    init() {
        this.domElement.tabIndex = -1;

        ['keydown', 'click'].forEach(eventType => {
            this.domElement.addEventListener(eventType, this, false);
        });
    }

    handleEvent(e) {
        let isEventConsumed = false;

        const eventType = e.type;
        if (eventType === 'keydown') {
            const code = e.keyCode;
            if (code === CODES.SPACE || code === CODES.ENTER) {
                dispatchClickEvent(e);
                isEventConsumed = true;
            }
        } else if (eventType === 'click') {
            // XXX
            console.log(`「${this.text}」が選択されました。`);
            this.parent.parent.close();
        }

        if (isEventConsumed) {
            e.stopPropagation();
            e.preventDefault();
        }
    }

    select() {
        this.domElement.focus();
    }

    get text() {
        return this.domElement.textContent;
    }
}

class Menu {
    constructor(domElement, parent) {
        this.domElement = domElement;
        this.parent = parent;
        this.init();
    }

    init() {
        // seperators and sub menus are not supported
        const menuitems = [];
        const els = this.domElement.querySelectorAll('[role="menuitem"]');
        for (let i = 0; i < els.length; i++) {
            menuitems.push(new MenuItem(els[i], this, i));
        }
        this.menuitems = menuitems;
        this.selectedIndex = 0;

        // Firefox doesn't support focusout
        // https://bugzilla.mozilla.org/show_bug.cgi?id=687787
        ['keydown', 'focusout'].forEach(eventType => {
            this.domElement.addEventListener(eventType, this, false);
        });
    }

    select(index) {
        const length = this.menuitems.length;
        index = (index + length) % length;
        this.menuitems[index].select();
        this.selectedIndex = index;
    }

    open(index) {
        this.domElement.style.display = 'block';
        this.select(index);
    }

    close() {
        this.domElement.style.display = 'none';
    }

    handleEvent(e) {
        let isEventConsumed = false;

        const eventType = e.type;
        if (eventType === 'keydown') {
            const code = e.keyCode;
            if (code === CODES.ESC) {
                this.parent.close();
                isEventConsumed = true;
            } else if (code === CODES.DOWN) {
                this.select(this.selectedIndex + 1);
                isEventConsumed = true;
            } else if (code === CODES.UP) {
                this.select(this.selectedIndex - 1);
                isEventConsumed = true;
            }
        } else if (eventType === 'focusout') {
            if (!e.relatedTarget || !this.isSelfOrAncestor(e.relatedTarget, this.domElement)) {
                this.parent.close();
            }
        }

        if (isEventConsumed) {
            e.stopPropagation();
            e.preventDefault();
        }
    }

    isSelfOrAncestor(el, base) {
        while (el) {
            if (el === base) {
                return true;
            }
            el = el.parentNode;
        }
        return false;
    }
}


class Btn {
    constructor(domElement, parent) {
        this.domElement = domElement;
        this.parent = parent;
        this.init();
    }

    init() {
        ['keydown', 'click'].forEach(eventType => {
            this.domElement.addEventListener(eventType, this, false);
        })
    }

    focus() {
        this.domElement.focus();
    }

    handleEvent(e) {
        let isEventConsumed = false;

        const eventType = e.type;
        if (eventType === 'click') {
            this.parent.open(0);
        } else if (eventType === 'keydown') {
            const code = e.keyCode;
            if (code === CODES.SPACE || code === CODES.ENTER) {
                dispatchClickEvent(e);
                isEventConsumed = true;
            } else if (code === CODES.DOWN) {
                this.parent.open(0);
                isEventConsumed = true;
            } else if (code === CODES.UP) {
                this.parent.open(-1);
                isEventConsumed = true;
            }
        }

        if (isEventConsumed) {
            e.stopPropagation();
            e.preventDefault();
        }
    }
}

class BtnMenu {
    constructor(el) {
        this.el = el;
        this.init();
    }

    init() {
        const btn = this.el.querySelector('button');
        const menu = this.el.querySelector('[role="menu"]');

        this.btn = new Btn(btn, this);
        this.menu = new Menu(menu, this);

        this.menu.close();
    }

    open(index) {
        this.menu.open(index);
    }

    close() {
        this.menu.close();
        this.btn.focus();
    }
}


const els = document.querySelectorAll('.btn-menu');
for (let i = 0; i < els.length; i++) {
    new BtnMenu(els[i]);
}
})();
