(function () {
'use strict';

const KEYCODE_2_DIRECTION = {
    37: 'LEFT',
    38: 'UP',
    39: 'RIGHT',
    40: 'DOWN'
}

class Day {
    constructor(calendar, domElement, index) {
        this.calendar = calendar;
        // (注:td要素)
        this.domElement = domElement;
        this.index = index;

        this.init();
    }

    init() {
        this.focusable = Day.wrapElement(this.domElement, document.createElement('button'));
        // XXX
        const id = `cell-${this.index}`;
        this.focusable.id = id;
        this.domElement.setAttribute('aria-labelledby', id);

        this.deselect();

        // td要素（this.domElement）がクリックされた場合の処理
        this.domElement.addEventListener('click', e => {
            this.calendar.updateSelection(this);
        }, false);

        // 矢印キーの処理
        this.domElement.addEventListener('keydown', e => {
            // 選択されていない日はフォーカスがあたらないのでkeydownイベントは発生しないはず
            if (!this.selected) {
                throw new Error('キーボード操作できるのは選択されている日のみ');
            }

            // (注:<kbd>Shift</kbd>などと矢印キーが同時に押されている場合は処理しない)
            if (e.shiftKey || e.ctrlKey || e.metaKey || e.altKey) {
                return;
            }

            // 押されたのが上下左右キーだったらフォーカスを移動
            const code = e.keyCode;
            if (code === 37 /* LEFT*/ || code === 38 /* UP */ ||
                code === 39 /* RIGHT*/ || code === 40 /* DOWN */) {
                this.calendar.moveSelection(KEYCODE_2_DIRECTION[code]);
            }
        }, false);
    }

    static wrapElement(parent, child) {
        while (parent.childNodes.length) {
            child.appendChild(parent.firstChild);
        }
        parent.appendChild(child);
        return child;
    }

    deselect() {
        this.selected = false;
        this.focusable.tabIndex = -1;
    }

    select(willFocus) {
        this.selected = true;
        this.focusable.tabIndex = 0;
        if (willFocus) {
            this.focusable.focus();
        }
    }

    get text() {
        const time = this.domElement.querySelector('time');
        return time.getAttribute('datetime');
    }
}

class Calendar {
    constructor(domElement) {
        // (注:table要素)
        this.domElement = domElement;
        this.selectedDay = null;
        this.init();
    }

    init() {
        this.days = [];

        const elements = this.domElement.querySelectorAll('tbody > tr >  td');
        for (let i = 0; i < elements.length; i++) {
            const element = elements[i];
            const day = new Day(this, element, i);
            this.days.push(day);
        }

        // 初期状態では最初の項目を選択
        this.updateSelection(this.days[0], false);

        this.initGrideRoles();
    }

    initGrideRoles() {
        this.domElement.setAttribute('role', 'grid');

        const selectorsAndRoles = [
            ['thead, tbody', 'rowgroup'],
            ['thead th', 'columnheader'],
            ['tr', 'row'],
            ['tbody td', 'gridcell']
        ]

        for (const [selector, role] of selectorsAndRoles) {
            const elements = this.domElement.querySelectorAll(selector);
            for (let i = 0; i < elements.length; i++) {
                elements[i].setAttribute('role', role);
            }
        }
    }

    updateSelection(newSelectedDay, willFocus) {
        if (this.selectedDay === newSelectedDay) {
            return;
        }

        if (this.selectedDay) {
            this.selectedDay.deselect();
        }

        this.selectedDay = newSelectedDay;
        this.selectedDay.select(willFocus);

        // XXX
        document.getElementById('result').textContent = this.selectedDay.text;
    }

    moveSelection(direction) {
        // 現在選択されている日の上下左右を探す処理
        const STRIDE = 7;
        const MAX = this.days.length;

        let index = this.selectedDay.index;

        if (direction === 'UP') {
            if (index > STRIDE) {
                index -= STRIDE;
            }
        } else if (direction === 'DOWN') {
            if (index + STRIDE < MAX) {
                index += STRIDE;
            }
        } else if (direction === 'LEFT') {
            if (index > 0) {
                index--;
            }
        } else if (direction === 'RIGHT') {
            if (index < MAX - 1) {
                index++;
            }
        }

        // 探した結果をnewSelectedDayに格納
        const newSelectedDay = this.days[index];
        this.updateSelection(newSelectedDay, true);
    }
}

new Calendar(document.querySelector('.calendar'));
})();
