(function() {
'use strict';

class Slide {
    constructor(domElement, index) {
        const id = `slide-${index}`;
        domElement.tabIndex = -1;
        domElement.id = id;
        this.title = domElement.querySelector('h1, h2, h3, h4, h5, h6').textContent;
        this.domElement = domElement;
    }

    deselect() {
        this.domElement.classList.remove('current');
    }

    select() {
        this.domElement.classList.add('current');
    }

    get id() {
        return this.domElement.id;
    }
}


class SlideShow {
    constructor(domElement) {
        this.domElement = domElement;
        this.selecteIndex = null;
        this.init();
    }

    initUI() {
        const ui = document.createElement('div');
        ui.classList.add('ui');

        const prevImg = document.createElement('img');
        prevImg.src = 'prev.png';
        prevImg.alt = '前へ';

        const prevButton = document.createElement('button');
        prevButton.classList.add('prev', 'button');
        prevButton.appendChild(prevImg);
        ui.appendChild(prevButton);
        this.prevButton = prevButton;

        const nextImg = document.createElement('img');
        nextImg.src = 'next.png';
        nextImg.alt = '次へ';

        const nextButton = document.createElement('button');
        nextButton.classList.add('next', 'button');
        nextButton.appendChild(nextImg);
        ui.appendChild(nextButton);
        this.nextButton = nextButton;

        const indicator = document.createElement('div');
        indicator.setAttribute('aria-live', 'polite');
        indicator.textContent = '現在のスライド:';

        const indicatorText = document.createElement('a');
        indicator.appendChild(indicatorText);
        ui.appendChild(indicator);
        this.indicatorText = indicatorText;

        this.domElement.appendChild(ui);
    }

    init() {
        this.initUI();

        this.slides = [];

        const slideElements = this.domElement.querySelectorAll('.slide');
        for (let i = 0; i < slideElements.length; i++) {
            const slide = new Slide(slideElements[i], i);
            this.slides.push(slide);
        }
        this.gotoSlide(0);

        window.addEventListener('keydown', (e) => {
            const code = e.keyCode;
            if (code === 37 /* Left */) {
                this.gotoPrevSlide();
            } else if (code === 39 /* Right */) {
                this.gotoNextSlide();
            }
        }, false);

        const hammer = new Hammer(this.domElement, {});
        hammer.on('swipe', (ev) => {
            const direction = ev.direction;
            if (direction === Hammer.DIRECTION_RIGHT) {
                this.gotoNextSlide();
            } else if (direction === Hammer.DIRECTION_LEFT) {
                this.gotoPrevSlide();
            }
        });

        this.nextButton.addEventListener('click', (e) => {
            this.gotoNextSlide();
        });

        this.prevButton.addEventListener('click', (e) => {
            this.gotoPrevSlide();
        });
    }


    get selected() {
        return (this.selectedIndex === null) ? null : this.slides[this.selectedIndex];
    }

    gotoPrevSlide() {
        this.gotoSlide(this.selectedIndex - 1);
    }

    gotoNextSlide() {
        this.gotoSlide(this.selectedIndex + 1);
    }

    gotoSlide(index) {
        if (index < 0 || this.slides.length - 1 < index) {
            return;
        }


        if (this.selected) {
            this.selected.deselect();
        }

        this.selectedIndex = index;
        this.selected.select();

        this.prevButton.disabled = index === 0;
        this.nextButton.disabled = (index == this.slides.length - 1);

        this.indicatorText.href = '#' + this.selected.id;
        this.indicatorText.textContent = this.selected.title;
    }

}

document.documentElement.classList.add('js');
new SlideShow(document.body);
})();
